!******************************************************************************
!
! File : metric.f90
!   by : Alan Tackett
!   on : 7/27/98
!  for : PAW Project
!
!  Only contains Gij IGij routines.     
! 
!******************************************************************************

Module metric

Use mathlib
Use paw_inout


Implicit NONE!!!!


Real              :: Gij(3,3)       !** Metric or g_(ij)
Real              :: IGij(3,3)       !** Inverse Metric or g^(ij)
 

!******************************************************************************
Contains
!******************************************************************************


!******************************************************************************
!
!  CalcGij - Calculates G_(ij) and g^(ij) for the transformation
!
!    Lattice - Lattice vectors
!
!******************************************************************************

Subroutine CalcGij(Lat)
  Real, Intent(IN) :: Lat(:,:)
  
  Integer :: i,j, k, Pivot(3)
  Real :: Work(9), Lattice(3,3)

  Gij = 0; IGij = 0;

Lattice = Lat


  Do i=1, 3
     Do j=1, 3
        Do k=1, 3
           Gij(i,j) = Gij(i,j) + Lattice(k,i)*Lattice(k,j)
        End Do
     End Do
  End Do

  Pivot = 0
  IGij = Gij
  Call DGETRF(3,3, IGij, 3, Pivot, i)
  If (i /= 0) then
     Write(Error_Unit,*) 'CalcGij: Error in DGETRF.  Error=',i
     Write(Error_Unit,*) 'CalcGij: Gij=',Gij
     Write(Error_Unit,*) 'CalcGij: IGij=',IGij
     Write(Error_Unit,*) 'CalcGij: Pivot = ',Pivot
     i = 0;
     j = 1/i;
  End If

  Call DGETRI(3, IGij, 3, Pivot, Work, 9, i)
  If (i /= 0) then
     Write(Error_Unit,*) 'CalcGij: Error in DGETRI.  Error=',i
     Write(Error_Unit,*) 'CalcGij: Gij=',Gij
     Write(Error_Unit,*) 'CalcGij: IGij=',IGij
     Write(Error_Unit,*) 'CalcGij: Pivot = ',Pivot
     i = 0;
     j = 1/i;
  End If


!  Write(Log_Unit, *) 'CalcGij:  Printing Gij Matrix'
!  Do i=1, 3
!     Write(Log_Unit, *) '               ',Gij(i,:) 
!  End Do

!  Write(Log_Unit,*) '               '
!  Write(Log_Unit, *) 'CalcGij:  Printing IGij Matrix'
!  Do i=1, 3
!     Write(Log_Unit, *) '               ',IGij(i,:) 
!  End Do

!  Write(Log_Unit,*) '               '
  Return

End Subroutine

End module
